################################################################################
# Created       : Martin Foster
# Modified      : 07-Sep-2003
################################################################################
#
# Stamp - Module part of Ethereal Realms, designed to handle timestamps
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package eRealms::Stamp;						# Package name


################################################################################
# Sub-Routines
################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $realm    = shift;					# Realm hash
	my $pref     = shift;					# Preffered position
	my $trgt     = shift;					# Target audience

	#####################
	# Program Area

	# Create internal hash
	my $self     = {
		DATE   => $database->GetDateFormat($param->{'ROOM'}),
		PARAM  => $param,
		PREF   => $pref,
		REALM  => $realm,
		TARGET => $trgt
 	 };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Header
#
# Final handling of the header in order to allow for someone to display the
# timestamp where necessary

sub Header
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $str      = shift;					# String to handle

	my $pre;						# Inserted
	my $post;						# Appended
	my $time     = $self->{'DATE'};				# Preffered position


	#####################
	# Program area	

	# Location is specified
	if ($str =~ /DATETIME/)
	{
		# Replace userdefined remove others
		$str =~ s/DATETIME/$time/gs;
		$str =~ s/SYSTIME//gs;
	}

	# Default
	else
	{
		# String handling
		$pre  = (defined($self->{'PRE'}))  ? $self->{'PRE'}  : '';
		$post = (defined($self->{'POST'})) ? $self->{'POST'} : '';

		# Replace with proper values
		$str =~ s/SYSTIME/$pre$time$post/gs;
	}


	# Return modification
	return $str;
}



#####################
# Date/Time stamp
#
# Functionality that allows the host to decide more or less where the date and
# time stamp to be placed.   Depending on the locale, different output will
# be generated.

sub Time
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $curr     = shift;					# Current position
	my $str      = shift;					# String to handle

	my $param    = $self->{'PARAM'};			# Parameter handle
	my $pref     = $self->{'PREF'};				# Preffered position
	my $realm    = $self->{'REALM'};			# Realm hash
	my $trgt     = $self->{'TARGET'};			# Target audience
	
	my $pesky;						# Handling of global posts


	#####################
	# Program area	

	# Process information
	if ($curr eq $pref)
	{
		# Determine audience
		# Simple date
		unless (($trgt ne 'all') 
		    && ($trgt ne 'system')
		    && ($trgt ne 'dice roll')
		    && ($trgt ne $realm->{'TxtSystem'}))
		{
			# Change format
			if ($pref eq 'bottom')
			{
				# Classical behaviour
				$str = "$str SYSTIME";

				# Append
				$self->{'PRE'}  = '(';
				$self->{'POST'} = ')';
			}
			elsif ($pref eq 'top')
			{
				# Alternate behaviour
				$str = "$str SYSTIME";

				# Append
				$self->{'POST'} = '<BR>';
			}
		}

		# Date with identification
		else
		{
			# Change format
			# Generate regular string
			if ($pref eq 'bottom')
			{
				# Pesky handling
				$pesky = (defined($param->{'PESKY'}))
				  ? " ($realm->{'TagPuppetPMRealm'}: $param->{'ROOM'})"
				  : '';

				# Set string
				$str = $str 
					. " ($realm->{'TagPuppetPMFrom'}: $param->{'CHAR'})"
					. " ($realm->{'TagPuppetPMTo'}: $param->{'OTARGET'})" . $pesky
					. " SYSTIME";

				# Append
				$self->{'PRE'}  = '(';
				$self->{'POST'} = ')';
			}

			# Generate stripped string
			else
			{
				# Pesky handling
				$pesky = (defined($param->{'PESKY'}))
				  ? " - $realm->{'TagPuppetPMRealm'}: $param->{'ROOM'}"
				  : '';

				# Set string
				$str = $str 
					. " SYSTIME"
					. " $realm->{'TagPuppetPMFrom'}: $param->{'CHAR'} - "
					. " $realm->{'TagPuppetPMTo'}: $param->{'OTARGET'}" . $pesky . "<BR>";

				# What we need to append
				$self->{'POST'} = ' - ';
			}
		}
	}


	# Return unchanged
	return $str;
}

# Alias for bottom
sub Bottom
{ return Time(shift, 'bottom', shift); } 

# Alias for top
sub Top
{ return Time(shift, 'top', shift); } 

# Alias for removed
sub Removed
{ return Time(shift, 'removed', shift); } 


1;								# Return true
