#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 27-May-2003
################################################################################
#
# Reset - Used to reset the password for a specific username
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Mail;						# Mail handling
use eRealms::Option;						# Option handling
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table module
use eRealms::Template;						# Template handler


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $send;							# Mail name
my $param;							# Parameter hash
my $table;							# Table handler
my $tmpl;							# Template handling

my $ban;							# Banning
my $crypt;							# Encypted passwords
my $login;							# Login name

my %config;							# Configuration hash
my %system;							# System hash


#################################################################################
# Program Area
#################################################################################

	
	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Establish connection
	$database->Connect();

	# Parameter handling
	$param = new eRealms::Param($database, $cgi);

	# Retreive parameters
	$param->GetParam();

	# Retreive hash
	$database->GetHashConfiguration(\%config);


	# Create instances
	$table = new eRealms::Table($cgi);
	$tmpl  = new eRealms::Template(\%config);


	# HTML Display
	print $cgi->header();
	$database->DocumentGetHeader();

	# Title
	$tmpl->ShowMain($config{'TagRstTitle'});


	# If no registration codes or logins		
	if (defined($param->{'LOGIN'}))
	{
		# Pull Email address
		my ($login, $ban, $adm, $sup) = $database->DataGet("SELECT
			 PuppeteerLogin,
			 PuppeteerBan,
			 PuppeteerAdminPrivs,
			 PuppeteerSuperPrivs
			FROM Puppeteer
			WHERE LOWER(PuppeteerLogin)=LOWER(?)",

		 $param->{'LOGIN'}
		);


		# Table top
		$table->MakeTop();

		# Verified
		if ((defined($login))
		 && ($ban eq 'no')
		 && ($adm eq 'no')
		 && ($sup eq 'no'))
		{
			# Evaluate
			eval
			{
				# Get hash
				$database->GetHashSystem(\%system);


				# Pull information
				my ($email) = $database->DataGetPuppeteerEmail($login);
				my ($name)  = $database->DataGetPuppeteerName($login);

				# Password reset
				my ($reset) = $database->DocumentGetReset();


				# Generate new password
				$crypt = md5_hex(rand());
				$crypt = substr($crypt, 0, 8);


				# Mail sending
				# Initiate handle
				$send = tie(*MAIL, 'eRealms::Mail', $database);

				# Initial setup
				$send->SetSubject("Your password for $login has been reset on $system{SetInfoServer}");
				$send->SetSearch(
					MMAIL => $email,
					MNAME => $name,
					MUSER => $login,
					MPASS => $crypt
				 );

				# Recipients
				$send->AddTo("$name <$email>");

				# Message
				print MAIL $reset;

				# Close and send 
				close(MAIL);
	

				# Update password
				$database->Write("UPDATE Puppeteer
					SET PuppeteerPassword=?
					WHERE PuppeteerLogin=?",

				  crypt($crypt, $crypt),
				  $login
				);


				# Commit
				$database->Commit();


				# Success
				$table->MakeSingle($tmpl->Pass('MsgRstPass', MMAIL=>$email)); 
			};

			# If block failed
			if ($@)
			{
				# Issue warning
				warn("Transaction aborted: $@");

				# Undo incomplete changes
				$database->Rollback();


				# Failure
				$table->MakeSingle($config{'MsgRstFail'});
			}
		}

		# Invalid access
		else
		{
			# Failure
			$table->MakeSingle($config{'MsgRstFail'});
		}

		# Close table
		$table->MakeBottom();
	}

	# No reset was requested
	else
	{ 
		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat()), "\n";

		# Table top and help
		$table->MakeTop();
		$table->MakeSingle($config{'HlpReset'});
		$table->MakeSingle($tmpl->PassTitle($config{'TagRstLogin'}));

		# Form filler
		$table->MakeValid(
			$cgi->textfield('LOGIN', '', 10, 10),
			$cgi->submit($config{'TxtReset'})
		 );
		$table->MakeBottom();


		# End form
		print $cgi->end_form();
	}

	# Footer
	$database->DocumentGetFooter();


################################################################################
# Sub-Routines
################################################################################
